import { EntityEquippableComponent, EntityOnFireComponent, EquipmentSlot, system, world } from "@minecraft/server";
import { DynamicLightingItems } from "./data";
const crouchingPlayers = {};
let players = [];
let onFireEntities = [];
export class DynamicLighting {
    static *job(player) {
        if (!player || !player.isValid())
            return;
        const entities = player?.dimension?.getEntities({
            location: player.location,
            maxDistance: 30,
            excludeTypes: ["item", "snowball", "armor_stand", "painting", "thrown_trident", "xp_orb", "xp_bottle", "splash_potion", "lingering_potion", "area_effect_cloud", "armor_stand"],
            excludeFamilies: ["minecart", "boat"]
        });
        if (entities)
            for (const entity of entities) {
                (() => {
                    for (let i = 0; i < onFireEntities.length; i++)
                        if (onFireEntities[i].id === entity.id)
                            return;
                    const comp = entity.getComponent(EntityOnFireComponent.componentId);
                    if (!comp)
                        return;
                    onFireEntities.push(entity);
                    let block = undefined;
                    try {
                        block = entity.dimension.getBlock(entity.getHeadLocation());
                    }
                    catch { }
                    if (!block || !block.isValid())
                        return;
                    const isWater = block.typeId.includes("water") && block.isLiquid;
                    if (!block.isAir && !isWater && block.typeId !== "ash_dynlts:light_block")
                        return;
                    block.setType("ash_dynlts:light_block");
                    block.setPermutation(block.permutation.withState("ash_dynlts:light_level", 8));
                })();
            }
        let block = undefined;
        try {
            block = player.dimension.getBlock(player.getHeadLocation());
        }
        catch { }
        if (!block || !block.isValid())
            return;
        const isWater = block.typeId.includes("water") && block.isLiquid;
        if (!block.isAir && !isWater && block.typeId !== "ash_dynlts:light_block")
            return;
        const comp = player.getComponent(EntityEquippableComponent.componentId);
        if (!comp)
            return;
        let lightLevel = 0;
        for (const slot in EquipmentSlot) {
            const item = comp.getEquipmentSlot(slot)?.getItem();
            const lightItem = DynamicLightingItems[item?.typeId ?? ""];
            if (!lightItem)
                continue;
            if ((isWater || block.isWaterlogged) && !lightItem.inWater)
                continue;
            if (lightItem.lightLevel > lightLevel)
                lightLevel = lightItem.lightLevel;
        }
        if (lightLevel <= 0)
            return;
        block.setType("ash_dynlts:light_block");
        block.setPermutation(block.permutation.withState("ash_dynlts:light_level", lightLevel));
    }
    static onCrouch(player) {
        let maxTime = 10; // time in ticks
        let tick = 0;
        let unCrouched = false;
        const interval = system.runInterval(() => {
            if (tick > maxTime || !player || !player.isValid()) {
                system.clearRun(interval);
                return;
            }
            if (!unCrouched) {
                if (!player.isSneaking)
                    unCrouched = true;
            }
            else {
                if (player.isSneaking) {
                    this.swapHands(player);
                    system.clearRun(interval);
                    return;
                }
            }
            tick++;
        }, 1);
    }
    static swapHands(player) {
        const comp = player.getComponent(EntityEquippableComponent.componentId);
        if (!comp)
            return;
        const mainhand = comp.getEquipmentSlot(EquipmentSlot.Mainhand);
        const offhand = comp.getEquipmentSlot(EquipmentSlot.Offhand);
        let offhandItem = offhand.getItem();
        let mainhandItem = mainhand.getItem();
        if (mainhandItem && (!DynamicLightingItems[mainhandItem.typeId] || !DynamicLightingItems[mainhandItem.typeId].offhand))
            return;
        if (offhandItem && (!DynamicLightingItems[offhandItem.typeId] || !DynamicLightingItems[offhandItem.typeId].offhand))
            return;
        if (mainhandItem) {
            player.runCommand(`replaceitem entity @s slot.weapon.offhand 0 ${mainhandItem.typeId} ${mainhandItem.amount}`);
            mainhand.setItem(undefined);
        }
        if (offhandItem) {
            player.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 ${offhandItem.typeId} ${offhandItem.amount}`);
            if (offhand.getItem()?.typeId === offhandItem.typeId)
                offhand.setItem(undefined);
        }
    }
    static initialize() {
        system.runInterval(() => {
            onFireEntities = [];
            for (const player of players)
                if (player && player.isValid())
                    system.runJob(this.job(player));
        }, 2);
        system.runInterval(() => {
            for (const player of players) {
                if (!player || !player.isValid())
                    continue;
                if (player.isSneaking) {
                    if (!crouchingPlayers[player.id]) {
                        this.onCrouch(player);
                        crouchingPlayers[player.id] = true;
                    }
                }
                else if (crouchingPlayers[player.id])
                    delete crouchingPlayers[player.id];
            }
        });
        system.runInterval(() => {
            players = world.getAllPlayers();
        }, 20);
        world.beforeEvents.worldInitialize.subscribe((data) => {
            data.blockComponentRegistry.registerCustomComponent("ash_dynlts:light_block", {
                onTick: (data) => {
                    const { block } = data;
                    block.setType("air");
                }
            });
        });
    }
}
